declare("test", {})
test.version = '0.2.6-a2'

-- Functions
test.init               = {}

-- Variables
test.config             = {}
test.config.major       = 'DevKit'
test.debug              = {}
test.debug.details      = (gkini.ReadInt(test.config.major, 'debug.details', 0) == 1) --activates extra details on return values from events
test.debug.events       = (gkini.ReadInt(test.config.major, 'debug.events', 0) == 1)
test.debug.print        = (gkini.ReadInt(test.config.major, 'debug.print', 1) == 1)
test.notesid            = 8378
local bool_map          = {[true] = 'ON', [false] = 'OFF'}

dofile('events.lua')
for i, v in ipairs(test.events) do test.events_ref.reverse_map[v[1]] = i end
dofile('functions.lua')

function test.init() --:OnEvent(event)
-- UnregisterEvent(self, "PLAYER_ENTERED_GAME")
	dofile('event_mgmt.lua')
	test.event_mgmt.init()
	dofile('new_table_viewer.lua')
	test.table_viewer.ui.build_dialog()
	dofile('func_int.lua')
	RegisterEvent(test.func_int.init, 'START')
	RegisterEvent(test.func_int.init, 'PLAYER_ENTERED_GAME')
	print(test.fontcolor..'DevKit Loaded [ v'..test.version..' ]')
end

RegisterUserCommand("test", function(_, args) 
	local fc = test.fontcolor

	args = args or {}
	for i, v in ipairs(args) do
		if (i == 2 and args[1] == 'table') then break end
		if (i == 3) then break end
		args[i] = v:lower()
	end

	local actions  = {}
	actions.default = function() print(fc..test.help.main) end

	actions.debug  = {
		details    = function()
		                 test.debug.details = not test.debug.details
		                 test.event_mgmt.ui.details.value = bool_map[test.debug.details]
		                 test.event_mgmt.ui.details:action()
		                 print(fc.."Debug Details: "..tostring(test.debug.details))
		             end,
		events     = function()
		                 test.debug.events = not test.debug.events
		                 test.event_mgmt.ui.debugevents.value = bool_map[test.debug.events]
		                 test.event_mgmt.ui.debugevents:action()
		                 print(fc.."Debug Events: "..tostring(test.debug.events))
		             end,
		help       = function() print(fc..test.help[args[1]]) end,
		print      = function()
		                 test.debug.print = not test.debug.print
		                 print(fc.."Debug Print: "..tostring(test.debug.print))
		             end,
		status     = function() 
		                 local stat_str = "Current status:\n"..
		                     " - Debug Details = "..tostring(test.debug.details).."\n"..
		                     " - Debug Events  = "..tostring(test.debug.events).."\n"..
		                     " - Debug Print   = "..tostring(test.debug.print)
		                 print(fc..stat_str) 
		             end,
	}
	actions.debug.default = actions.debug.help

	actions.events = {
		clear      = function()
		                 test.event_mgmt.ui.clearall:action()
		                 print(fc..'All events cleared.')
		             end,
		help       = function() print(fc..test.help[args[1]]) end,
		load       = function()
		                 test.event_mgmt.ui.clearall:action()
		                 test.event_mgmt.load_defaults()
		                 print(fc..'Default events loaded')
		             end,
		reg        = function(v) test.event_mgmt.manual_toggle('reg', v) end,
		save       = function() 
		                 test.event_mgmt.ui.save:action()
		                 print(fc..'Default events saved')
		             end,
		status     = function() test.event_mgmt.event_status() end,
		unreg      = function(v) test.event_mgmt.manual_toggle('unreg', v) end,
	}
	actions.events.default = function() ShowDialog(test.event_mgmt.ui.dialog) end

	actions.funcint = {
		add        = function(v) test.func_int.add(v) end,
		del        = function(v) test.func_int.del(v) end,
		delall     = function() test.func_int.delall() end,
		help       = function() print(fc..test.help[args[1]]) end,
		list       = function() test.func_int.list() end,
		tog        = function(v) test.func_int.toggle(v) end,
	}
	actions.funcint.default = actions.funcint.help

	actions.table  = {
		help       = function() print(fc..test.help[args[1]]) end,
	}
	actions.table.default = function(v) test.table_viewer.ui.dialog:Show(v) end


	-- Execute the desired action, as defined in actions
	if (args == nil) then
		actions.default()
	elseif (actions[args[1]]) then
		if (actions[args[1]][args[2]]) then 
			actions[args[1]][args[2]](args[3])
		else
			actions[args[1]].default(args[2])
		end
	else
		actions.default()
	end

end)

RegisterUserCommand("reload", function() print('Reloading...'); ReloadInterface() end)
--RegisterEvent(test.init, "PLAYER_ENTERED_GAME")

test.init()